@@ -40,6 +40,10 @@
     
     bdajax = {
         
+        // By default, we redirect to the login page on 403 error.
+        // That we assume at '/login'.
+        default_403: '/login',
+        
         // object for 3rd party binders
         binders: {},
         
@@ -77,19 +81,6 @@
             }
         },
         
-        ajaxerrors: {
-            timeout: 'The request has timed out. Pleasae try again.',
-            error: 'An error occoured while processing the request. Aborting.',
-            parsererror: 'The Response could not be parsed. Aborting.',
-            unknown: 'An unknown error occoured while request. Aborting.'
-        },
-        
-        ajaxerror: function(status) {
-            if (status == 'notmodified') { return; }
-            if (status == null) { status = 'unknown'; }
-            return bdajax.ajaxerrors[status];
-        },
-        
         parseurl: function(url) {
             var idx = url.indexOf('?');
             if (idx != -1) {
@@ -134,10 +125,14 @@
             }
             if (!config.type) { config.type = 'html'; }
             if (!config.error) {
-                config.error = function(request, status) {
-                    var err = bdajax.ajaxerror(status);
-                    if (err) { bdajax.error(err); }
-                }
+                config.error = function(req, status, exception) {
+                    if (parseInt(status, 10) === 403) {
+                        window.location.pathname = bdajax.default_403;
+                    } else {
+                        bdajax.error(
+                            '<strong>' + status + '</strong> ' + exception);
+                    }
+                };
             }
             if (!config.cache) { config.cache = false; }
             var wrapped_success = function(data, status, request) {
@@ -145,7 +140,9 @@
                 bdajax.spinner.hide();
             }
             var wrapped_error = function(request, status, error) {
-                config.error(request, status, error);
+                config.error(request,
+                             request.status || status,
+                             request.statusText || error);
                 bdajax.spinner.hide(true);
             }
             bdajax.spinner.show();
@@ -163,9 +160,6 @@
             config.params['bdajax.action'] = config.name;
             config.params['bdajax.mode'] = config.mode;
             config.params['bdajax.selector'] = config.selector;
-            var error = function(req, status, exception) {
-                bdajax.error(exception);
-            };
             bdajax.request({
                 url: bdajax.parseurl(config.url) + '/ajaxaction',
                 type: 'json',
@@ -173,26 +167,51 @@
                 success: function(data) {
                     if (!data) {
                         bdajax.error('Empty response');
-                    }
-                    var mode = data.mode;
-                    var selector = data.selector;
-                    if (mode == 'replace') {
-                        $(selector).replaceWith(data.payload);
-                        var context = $(selector);
-                        if (context.length) {
-                            context.parent().bdajax();
-                        } else {
-                            $(document).bdajax();
+                        bdajax.spinner.hide();
+                    } else {
+                        var mode = data.mode;
+                        var selector = data.selector;
+                        if (mode == 'replace') {
+                            $(selector).replaceWith(data.payload);
+                            var context = $(selector);
+                            if (context.length) {
+                                context.parent().bdajax();
+                            } else {
+                                $(document).bdajax();
+                            }
+                        } else if (mode == 'inner') {
+                            $(selector).html(data.payload);
+                            $(selector).bdajax();
                         }
-                    } else if (mode == 'inner') {
-                        $(selector).html(data.payload);
-                        $(selector).bdajax();
+                        bdajax.continuation(data.continuation);
                     }
-                },
-                error: error
+                }
             });
         },
         
+        continuation: function(actions) {
+            if (!actions) {
+                return;
+            }
+            bdajax.spinner.hide();
+            var action, target;
+            for (var idx in actions) {
+                action = actions[idx];
+                if (action.type == 'action') {
+                    target = bdajax.parsetarget(action.target);
+                    bdajax.action({
+                        url: target.url,
+                        params: target.params,
+                        name: action.name,
+                        mode: action.mode,
+                        selector: action.selector
+                    });
+                } else if (action.type == 'event') {
+                    bdajax.trigger(action.name, action.selector, action.target);
+                }
+            }
+        },
+        
         trigger: function(name, selector, target) {
             var evt = $.Event(name);
             evt.ajaxtarget = bdajax.parsetarget(target);
@@ -239,6 +258,9 @@
                     var overlay = this.getOverlay();
                     $('.message', overlay).html(message);
                 },
+                onLoad: function() {
+                    elem.find('button:first').focus();
+                },
                 oneInstance: false,
                 closeOnClick: false,
                 top:'20%'
@@ -260,7 +282,7 @@
         
         warning: function(message) {
             $("#ajax-message .message").removeClass('error warning info')
-                                            .addClass('warning');
+                                       .addClass('warning');
             bdajax.message(message);
         },
         
@@ -327,6 +349,7 @@
         },
         
         _defs_to_array: function(str) {
+            // XXX: if space in selector when receiving def str, this will fail
             var arr;
             if (str.indexOf(' ') != -1) {
                 arr = str.split(' ');
@@ -337,4 +360,4 @@
         }
     };
 
-})(jQuery);
\ No newline at end of file
+})(jQuery);
